/*
@author Valentin CARRUESCO aka Idleman <idleman@idleman.fr>
@url http://blog.idleman.fr
@licence CC cc-by-nc-sa http://creativecommons.org/licenses/by-nc-sa/3.0/
@description 
============
Ce code permet l'ouverture d'une porte à l'aide d'un simple code bluetooth
transmis par votre téléphone/tablette etc...


@hardware
Vous devez envoyer ce code sur un atemga 328 (ou arduino quelquonque contenant l'atmega 328)
relié à un composant bluetooth (ici bolutek uk350), à une carte relais (elle meme reliée a la gache electrique de votre porte)
et à un bouton poussoir (pour l'ouverture manuelle coté interieur de la porte)

@usage
======
Installez l'application Yana bluetooth sur votre téléphone/tablette et configurez la
avec le même "PASSWORD" que la constante plus bas

*/

#include <SoftwareSerial.h>
//Pin TX (tranmsission) du composant bluetooth
#define BLUETOOTH_TRANSMIT_PIN 9 
//Pin RX (réception) du composant bluetooth
#define BLUETOOTH_RECEIVE_PIN 8
//Pin data du relais 
#define RELAY_PIN 7
//Pin du bouton d'ouverture manuel
#define BUTTON_PIN 11
//Délais d'ouverture (en ms)
#define OPEN_DELAY 5000
//Mot de passe d'ouverture (à renseigner dans l'application android)
#define PASSWORD "idle"

//Tableau de caracteres utilisé pour stocker le password
char allowed[4];
//Iterateur pour le stockage des char bluetooth recus
int j = 0;
// caractere de stockage temporaire
char tmp;
// tableau de caractere du code bluetooth recu (à comparer avec allowed)
char code[5]; 
//booleen indiquant le début d'un code
boolean startcode;

int buttonState = 0;         // variable for reading the pushbutton status

long debounceDelay = 50;    // the debounce time; increase if the output flickers

//Classe de gestion bluetooth (initialisée avec les pin tx/rx confuguré plus haut)
SoftwareSerial BluetoothSerial(BLUETOOTH_TRANSMIT_PIN, BLUETOOTH_RECEIVE_PIN);

//Fonction d'initialisation
void setup() {
     //Initialisation du pin relais en sortie
     pinMode(RELAY_PIN, OUTPUT);
     //Initialisation du pin boutton en entrée
     pinMode(BUTTON_PIN, INPUT);
     //Initialisation de la console
     Serial.begin(9600);
     //Initialisation du bluetooth
     BluetoothSerial.begin(9600);
     //Stockage du mot de passe correct dans un tableau de caracteres (le fameux "allowed")
     String(PASSWORD).toCharArray(allowed, 5);
     Serial.write("Porte initialisée");
}

//Boucle d'execution
void loop() {
     
     buttonState = digitalRead(BUTTON_PIN);

  if (buttonState == HIGH) {     
   unlock();
  }
     //Si une info bluetooth est captée
     if (BluetoothSerial.available()){
          // Tant qu'une lettre est transmise
          while(BluetoothSerial.available()){
               //On récupere la lettre
               tmp = (char) BluetoothSerial.read();
               //Si la lettre est '*' c'est qu'un code d'identification vas suivre, on met donc startcode à "true"
               if(tmp == '*') startcode = true;
               //Si startcode est à true et qu'on à pas encore récuperé les 4 lettres du code
               if(startcode && j<5){
                    //on stocke la lettre dans le tableau de comparaison
                    code[j] = tmp;  
                    //on ajoute +1 à l'iterateur pour stocker la prochaine lettre
                    j++;
               }
          }
        
          //Une fois le message bluetooth transmis, si un code à été transmis (si le caractere '*' à été trouvé)
          if(startcode){
               Serial.write("\nTest du code : ");
               Serial.write(code);
               //On compare le code trouvé au code autorisé avec la fonction checkcode définie plus bas
               if (checkCode()){
                    //Si le code est bon
                    Serial.write("\nReussite");
                    //Envois d'un message de confirmation au téléphone/tablette etc... connecté
                    BluetoothSerial.write("msg:code correct");
                    //Ouverture de la porte via la fonction unlock définie plus bas
                    unlock();
                    //réinitialisation des variables
                    startcode = false ;
                    memset(code, 0, sizeof(code));
                    j=0;
               }else{
                    //Si le code est incorrect,envois d'un emssage d'erreur de code à la concole et au téléphone/tablette etc... connecté
                     Serial.write("\nEchec");
                     BluetoothSerial.write("msg:code incorrect");
               }
          }
     }
}

//Fonction de comparaison entre le code recu et le code autorisé, retourne "true" si le code est bon, false dans le cas contraire
boolean checkCode(){
     return (code[0]=='*' && code[1]==allowed[0] && code[2]==allowed[1] && code[3]==allowed[2] && code[4]==allowed[3]);
}

//Fonction d'ouverture de la porte, ouvre la porte pendant un nombre de milisecondes définis dans les constantes en haut de fichier (5000ms = 5s)
void unlock(){
     Serial.write("\n Ouverture de la porte");
     BluetoothSerial.write("1");
     digitalWrite(RELAY_PIN,1);
     delay(OPEN_DELAY);
     BluetoothSerial.write("0");
     Serial.write("\n Fermeture de la porte");
     digitalWrite(RELAY_PIN,0);
}


